# -*- coding: utf-8 -*-

from osv import osv, fields
from datetime import datetime, timedelta

class apem_calendar_event(osv.osv):
    '''
    Extends the basic calendar event to add support for more timezones.
    It solves an issue when trying to accept an event invitation into the openerp caldav calendar.
    '''
    _inherit = "calendar.event"
    
    def create(self, cr, uid, vals, context=None):
        '''
        Extends the create method for compatibility with windows timezones.
        TODO: For now just romance support is added.
              We should add support for all of the windows zones
        '''
        if vals.get('vtimezone', '') == 'romance':
            vals['vtimezone'] = 'europe/paris'
        return super(apem_calendar_event,self).create(cr,uid,vals,context=context)
    
apem_calendar_event()


class apem_super_calendar(osv.osv):
    '''
    Extends supercalendar module to add a fiew fields
    '''
    _inherit = 'super.calendar'
    
    def _format_organizer(self, cr, uid, ids, field_name, arg, context=None):
        '''
        The organizer needs no be of the form: "name <email>" for Thunderbird caldav to work correctly.
        This function returns a field formatted this way.
        '''
        res = {}
        
        for cur_id in ids:
            cur_event = self.browse(cr, uid, cur_id, context=context)
            cur_user = cur_event.user_id
            
            # If no name is provided for the user, fallback to the login
            n = cur_user.name or cur_user.login
            m = cur_user.user_email or ''
            res[cur_id] = "%s <%s>" % (n,m)
            
        return res
    
    def _consolidate_name(self, cr, uid, ids, field_name, arg, context=None):
        '''
        For better readability, we prefix the event title with the name of the user.
        It's a must-have when the calendars of everyone are merged.
        '''
        res = {}
        
        for cur_id in ids:
            cur_event = self.browse(cr, uid, cur_id, context=context)
            cur_user = cur_event.user_id
            res[cur_id] = "[%s] %s" % (cur_user.login, cur_event.name)
            
        return res
    
    def _compute_end_date(self, cr, uid, ids, field_name, arg, context=None):
        '''
        caldav requires an end date while supercalendar uses a "duration" field.
        This function computes a valid end date.
        '''
        res = {}
        
        for cur_id in ids:
            cur_event = self.browse(cr, uid, cur_id, context=context)
            
            #duration is optional. If not set, assume it's a 1hour event (enough to make it visible)
            dur = cur_event.duration or 1
            res[cur_id] = (datetime.strptime(cur_event.date_start,'%Y-%m-%d %H:%M:%S') + timedelta(hours=dur)).strftime("%Y-%m-%d %H:%M:%S")
            
        return res
    
    _columns  = {
        #expose the database ID
        'id': fields.integer('ID', readonly=True),
        
        #This field was mandatory in caldav so I provide it even if it'll always be empty (no recurrent event allowed in super-calendar)
        'recurrent_uid': fields.integer('Recurrent ID'),
        
        #calculated fields for caldav-compliance
        'organizer': fields.function(_format_organizer, type='char', method=True, string='Organizer'),
        'consolidated_name': fields.function(_consolidate_name, type='char',method=True, string='Consolidated name'),
        'date_end': fields.function(_compute_end_date, type='datetime',method=True, string='End date'),
        #FIXME:  This should be improved by adding a timezone to the event.
        #        Otherwise, each time a user travel and changes his timezone, his events are messed up.
        'vtimezone':fields.related("user_id", "context_tz", type='char', string='Timezone', store=False)
    }

apem_super_calendar()