/**
	This module adds client-side javascript that will ask confirmation to the user before leaving a modified form.
	This should hopefully prevent misclicks causing data loss.
**/

openerp.apem_confirm_unsaved = function(openerp) {
	var QWeb = openerp.web.qweb,
	    _t = openerp.web._t;
	
	/**
		Checks is the user isn't loosing any unsaved data or if this loss is on purpose
	**/
	var check_leaving_will = function() {
		//@author: lp:~openerp-dev/openerp-web/trunk-form-notifier-vda
		var action_manager = this._parent.action_manager;
        if(action_manager.inner_viewmanager && action_manager.inner_viewmanager.active_view == 'form') {
            var active_view = action_manager.inner_viewmanager.active_view,
                controller = action_manager.inner_viewmanager.views[active_view].controller;
            return controller.can_be_discarded();
        }
        return true;
	};
	
	openerp.web.Header =  openerp.web.Header.extend({
		/**
			Overrides onunload event to prevent the user from loosing data when leaving the page.
		**/
		init: function(parent){
			this._parent = parent;
			
	    	self = this;
			$(window).bind('beforeunload', function() {
				//@author: lp:~openerp-dev/openerp-web/trunk-form-notifier-vda
				var action_manager = self._parent.action_manager;
		        if(action_manager.inner_viewmanager && action_manager.inner_viewmanager.active_view == 'form') {
		            var active_view = action_manager.inner_viewmanager.active_view,
		                controller = action_manager.inner_viewmanager.views[active_view].controller;
					
		            if(controller.$element.is('.oe_form_dirty'))
		            	return "You have unsaved changes.\nAre you sure you want to leave?";
				}
			});
			
			return this._super(parent);
		},
		/**
			Add data loss check when the user clicks on the "logout" button
		**/
	    on_logout: function() {
	    	if (check_leaving_will.apply(this, arguments))
            	return this._super.apply(this, arguments);
	    	return false;
	    },
		/**
			Add data loss check when the user clicks on the "preferences" button
			TODO:	Just looking at the preferences isn't harmful.
					The event should be placed on the 'save' button that causes the page to reload.
		**/
	    on_preferences: function(){
	    	if (check_leaving_will.apply(this, arguments))
            	return this._super.apply(this, arguments);
	    	return false;
	    },
		/**
			Add data loss check when the user clicks on the shortcuts.
			The inner function isn't exposed and named so it can't be extended.
			This copy-pastes a lot of the parent function instead.
		**/
	    shortcut_load :function(){
	        var self = this,
	            sc = self.session.shortcuts,
	            shortcuts_ds = new openerp.web.DataSet(this, 'ir.ui.view_sc');
	        // TODO: better way to communicate between sections.
	        // sc.bindings, because jquery does not bind/trigger on arrays...
	        if (!sc.binding) {
	            sc.binding = {};
	            $(sc.binding).bind({
	                'add': function (e, attrs) {
	                    shortcuts_ds.create(attrs, function (out) {
	                        $('<li>', {
	                            'data-shortcut-id':out.result,
	                            'data-id': attrs.res_id
	                        }).text(attrs.name)
	                          .appendTo(self.$element.find('.oe-shortcuts ul'));
	                        attrs.id = out.result;
	                        sc.push(attrs);
	                    });
	                },
	                'remove-current': function () {
	                    var menu_id = self.session.active_id;
	                    var $shortcut = self.$element
	                        .find('.oe-shortcuts li[data-id=' + menu_id + ']');
	                    var shortcut_id = $shortcut.data('shortcut-id');
	                    $shortcut.remove();
	                    shortcuts_ds.unlink([shortcut_id]);
	                    var sc_new = _.reject(sc, function(shortcut){ return shortcut_id === shortcut.id});
	                    sc.splice(0, sc.length);
	                    sc.push.apply(sc, sc_new);
	                    }
	            });
	        }
	        return this.rpc('/web/session/sc_list', {}, function(shortcuts) {
	            sc.splice(0, sc.length);
	            sc.push.apply(sc, shortcuts);
	
	            self.$element.find('.oe-shortcuts')
	                .html(QWeb.render('Shortcuts', {'shortcuts': shortcuts}))
	                .undelegate('li', 'click')
	
	                .delegate('li', 'click', function(e) {
		    			if (check_leaving_will.apply(self, arguments)){
		                    e.stopPropagation();
		                    var id = $(this).data('id');
		                    self.session.active_id = id;
		                    self.rpc('/web/menu/action', {'menu_id':id}, function(ir_menu_data) {
		                        if (ir_menu_data.action.length){
		                            self.on_action(ir_menu_data.action[0][2]);
		                        }
		                    });
		                }
	                });
	        });
		}
	});
		
    openerp.web.Menu = openerp.web.Menu.extend({
		init: function(parent, element_id, secondary_menu_id) {
			this._parent = parent;
			return this._super(parent, element_id, secondary_menu_id);
		},
		/**
			Add data loss check when the user clicks on a menu item
			TODO: Parent menu items shouldn't trigger the warning. Only leaves should.
		**/
        on_menu_click: function() {
	    	if (check_leaving_will.apply(this, arguments))
            	return this._super.apply(this, arguments);
	    	return false;
        }
    });
    
    openerp.web.ViewManager.include({
		/**
			Add data loss check when the user clicks on a button to switch the view mode
		**/
        on_mode_switch: function(view_type, no_store) {
			//@author: lp:~openerp-dev/openerp-web/trunk-form-notifier-vda
            if(this.active_view === 'form') {
               var controller = this.views[this.active_view].controller;
               var def = $.Deferred();
               if(!no_store && !controller.can_be_discarded()) {
                    return def.reject();
                }
            }
            return this._super.apply(this, arguments);
        }
    });
};