# -*- coding: utf-8 -*-

def _get_duplicates_ids(table, cr, uid, context=None, fields_list=None):
    '''
        @param table: The table we want to fetch for duplicates.
        @param cr: The database cursor.
        @param uid: The user performing the action.
        @param context: Context variables.
        @param fields_list: List of the fields on which we have to check for duplicates.
                            If none is provided, consider we have to test on all of them.
        @return: a list of the ids of the duplicate records for the given table
    '''
    
    if not fields_list:
        fields_list = []        # contains just the name of the field
        valid_fields_list = []  # contains the string casting/defaulting the field into a valid value
        
        #FIXME: access to a "private" attribute. Replace with a getter(here and multiple following calls)
        for c in table._all_columns:
            '''
                Loops through all of the fields in the table.
                Only retains the one whose type is known and that are actually stored in the database.
                
                Defaults the value to "0" casted to whatever the type is.
                This is required because in SQL:   (NULL == NULL)  is false    while   (0 == 0)  is true.
            ''' 
            if 'column' in dir(table._all_columns[c]):
                if 'store' not in dir(table._all_columns[c].column) or table._all_columns[c].column.store == True:
                    if table._all_columns[c].column._type == "integer" or  \
                            table._all_columns[c].column._type == "char" or \
                            table._all_columns[c].column._type == "boolean" or \
                            table._all_columns[c].column._type == "float" or  \
                            table._all_columns[c].column._type == "text":
                        fields_list.append(table._all_columns[c].name)
                        valid_fields_list.append("coalesce(\""+table._all_columns[c].name+"\",CAST ('0' as " + table._all_columns[c].column._type + "))")    
                    
                    if table._all_columns[c].column._type == "many2one":
                        fields_list.append(table._all_columns[c].name)
                        valid_fields_list.append("coalesce(\""+table._all_columns[c].name+"\",CAST ('0' as integer))")
    
    else:
        valid_fields_list = fields_list
                        
    fields = "\"" +  "\",\"".join(fields_list) + "\""
    valid_fields = ','.join(valid_fields_list)
    
    sql_req = """
        SELECT id FROM %s where (%s) in (
            SELECT %s
            FROM %s
            GROUP BY %s
            HAVING COUNT(*) > 1
        )
        """ % (table._table,valid_fields,valid_fields,table._table,fields)
        
    cr.execute(sql_req)
    return [x[0] for x in cr.fetchall()]

def is_duplicate(table, cr, uid, ids, field_name, args, context=None, fields_list=None):  
    '''
        For each given id in the given table, checks if there exists a duplicate.
    '''
    
    dup_ids = _get_duplicates_ids(table, cr, uid, context=context, fields_list=fields_list)
        
    res = {}
    for i in ids:
        '''
            We need to include all the ids in the result returned not just the duplicates.
            That's why we need to restart from the initial "ids" instead of just "dup_ids"
        '''
        if i in dup_ids:
            res[i] = True
        else:
            res[i] = False
            
    return res

def name_not_uniq(table, cr, uid, ids, field_name, args, context=None):
    return is_duplicate(table, cr, uid, ids, field_name, args, context=None, fields_list=['name'])
    
    
def search_is_duplicate(obj, cr, uid, table, field_name, args, context=None, fields_list=None):
    '''
        @return: An equivalent domain based on the id instead of the "function" field that isn't stored.
    '''
    dup_ids = _get_duplicates_ids(table, cr, uid, context=context, fields_list=fields_list)
    dom = []
    
    for cur_d in args:
        if cur_d[0] == field_name:
            if (cur_d[1] == '=' and cur_d[2] == True) or (cur_d[1] == '!=' and cur_d[2] == False):
                dom.append(('id', 'in', dup_ids))
            elif (cur_d[1] == '=' and cur_d[2] == False) or (cur_d[1] == '!=' and cur_d[2] == True):
                dom.append(('id', 'not in', dup_ids))
        else:
            dom.append(cur_d)

    return dom

def search_name_not_uniq(table, cr, uid, ids, field_name, args, context=None):
    return search_is_duplicate(table, cr, uid, ids, field_name, args, context=None, fields_list=['name'])