# -*- coding: utf-8 -*-

from osv import fields, osv
import time
from tools.translate import _

from apem_product_extensions.apem_product_series import apem_product_series

class apem_cpi_form(osv.osv):
    _name="crm.form.cpi"
    _inherits={"apem.form.abstract" : "abstract_form_id"}
    _description="Form FC00001N for a custom product inquiry"
    
    def _get_SALES_group(self, cr, uid, context=None):
        categ = self.pool.get('ir.module.category').search(cr, uid, [('name', '=', 'Sales Management')], context=context)
        groups = self.pool.get('res.groups').search(cr, uid, [('name', '=', 'User - All Leads'), ('category_id', "in", categ)], context=context)
        return groups
    
    def _get_PM_group(self, cr, uid, context=None):
        categ = self.pool.get('ir.module.category').search(cr, uid, [('name', '=', 'Sales Management')], context=context)
        groups = self.pool.get('res.groups').search(cr, uid, [('name', '=', 'Manager'), ('category_id', "in", categ)], context=context)
        return groups
        
    def _get_PM_members(self, cr, uid, context=None):
        groups = self._get_PM_group(cr, uid, context=context)
        if len(groups) == 1:
            return [(p.id, p.name) for p in self.pool.get('res.groups').browse(cr,uid,groups[0]).users]
        return []
    
    def _get_ENG_group(self, cr, uid, context=None):
        categ = self.pool.get('ir.module.category').search(cr, uid, [('name', '=', 'Sales Management')], context=context)
        groups = self.pool.get('res.groups').search(cr, uid, [('name', '=', 'Engineering'), ('category_id', "in", categ)], context=context)
        return groups
       
    def _get_ENG_members(self, cr, uid, context=None):
        groups = self._get_ENG_group(cr, uid, context=context)
        if len(groups) == 1:
            return [(p.id, p.name) for p in self.pool.get('res.groups').browse(cr,uid,groups[0]).users]
        return []
    
    def _get_METH_group(self, cr, uid, context=None):
        categ = self.pool.get('ir.module.category').search(cr, uid, [('name', '=', 'Sales Management')], context=context)
        groups = self.pool.get('res.groups').search(cr, uid, [('name', '=', 'Methods'), ('category_id', "in", categ)], context=context)
        return groups
        
    def _get_METH_members(self, cr, uid, context=None):
        groups = self._get_METH_group(cr, uid, context=context)
        if len(groups) == 1:
            return [(p.id, p.name) for p in self.pool.get('res.groups').browse(cr,uid,groups[0]).users]
        return []
    
    _columns={
              #link
              'sales_orders': fields.many2many('sale.order', 'res_so_cpi_rel', 'cpi_id', 'so_id', 'Ordered in'),
              
              #commercial part
              'demand_date': fields.date('Demand date', readonly=True),
              
              'project_lifetime': fields.char('Project life time',size=64),
              'application': fields.char('Application',size=256),
              'cpi_type':fields.selection([('1','Level 1 : No product modification'),
                                           ('2','Level 2 : Minor product modification'),
                                           ('3','Level 3 : Heavy product modification'),
                                           ('4','Level 4 : Customized product')], 'CPI type'),
                            
              'series': fields.many2one('product.series','Series'),
              'part_number': fields.char('Part number',size=64),
              'global_potential': fields.integer('Global potential', help="quantity/year"),   
              'accessible_potential': fields.integer('Accessible potential', help="quantity/year"),
              'precision_required': fields.selection((('estimation','Estimation'), ('budget','Budget'), \
                                             ('real','Real')), 'Precision', \
                                            help="How precise you need the price estimation to be"),
              'details': fields.text('Details of inquiry'),

              'target_price': fields.float('Target price', digits=(10,4), required=True),
              'customer_price': fields.float('For your customer', digits=(10,4)),
              'net_price': fields.float('Your net price', help="ExW", digits=(10,4)),
              'tooling_participation': fields.boolean('Customer participation to tooling costs'),
              'requested_answer_date': fields.date('Requested answer date'),
              
              #PM answer
              'decision':fields.selection([ ('ok','OK for R&D'), ('other','Recommendation of other product'), ('no','Decline')], \
                                          'Decision',write=['base.group_sale_manager'], read=['base.group_sale_salesman']),
              'answer': fields.text('Answer to the specific inquiry',write=['base.group_sale_manager'], read=['base.group_sale_salesman']),
             
              #workflow
              'in_charge_PM': fields.many2one('res.users', 'PM in charge', help="Product manager in charge of this demand",
                                              selection=_get_PM_members),
              'in_charge_Engineering': fields.many2one('res.users', 'Engineer in charge', help="Engineer in charge of this demand",
                                              selection=_get_ENG_members),
              'in_charge_Methods': fields.many2one('res.users', 'Methods member in charge', help="Employee of the methods department in charge of this demand",
                                              selection=_get_METH_members),
              
              #attachments
              'attachments': fields.many2many('ir.attachment', 'res_cpi_attachments_rel', 'cpi_id', 'attachment_id', 'Attachments'),
              
              #hierarchy
              'parent_form': fields.many2one('crm.form.cpi', 'Duplicated from', readonly=True),
              'children_forms': fields.one2many('crm.form.cpi', 'parent_form', 'Children', readonly=True),
              }
    _defaults={
              'name':  lambda self,cr,uid,context={}: self.pool.get('ir.sequence').get(cr, uid, 'crm.form.cpi'), 
              'demand_date': lambda *a: time.strftime('%Y-%m-%d'),
              }
    
    def _get_project_id(self, cr, uid):
        projectManager = self.pool.get('project.project')
        res = projectManager.search(cr,uid,[('name', '=', 'CPI forms')])
        if len(res) == 1:
            return res[0]
        else:
            return False
    
    def fields_get(self, cr, uid, fields=None, context=None):
        """Sets the field 'decision' and 'answer' as readonly if the user doesn't have the right to write on them"""
        user = self.pool.get('res.users').browse(cr, uid, uid)
        res = super(apem_cpi_form,self).fields_get(cr, uid, fields, context)
        if 'decision' in res and 'answer' in res:
            res['decision']['readonly'] = True
            res['answer']['readonly'] = True
            for cur_group in user.groups_id:
                if cur_group.full_name == 'Sales Management / Manager':
                    del res['decision']['readonly']
                    del res['answer']['readonly']
            
        if 'in_charge_PM' in res:
            res['in_charge_PM']['domain'] = "[('id','in',["+",".join([str(m[0]) for m in self._get_PM_members(cr,uid)]) + "])]"
        if 'in_charge_Engineering' in res:
            res['in_charge_Engineering']['domain'] = "[('id','in',["+",".join([str(m[0]) for m in self._get_ENG_members(cr,uid)]) + "])]"
        if 'in_charge_Methods' in res:
            res['in_charge_Methods']['domain'] = "[('id','in',["+",".join([str(m[0]) for m in self._get_METH_members(cr,uid)]) + "])]"
            
        return res
    
    def create(self, cr, uid, data, context=None):
        '''tasks_ids and attachments need some special values injection based on the form id.
           To achieve this, we remove them from the initial creation and call a "write" on the created form afterwards'''
        
        full_data = data.copy()
        if 'tasks_ids' in data:
            del data['tasks_ids']
        if 'attachments' in data:
            del data['attachments']
            
        cur_id = super(apem_cpi_form, self).create(cr, uid, data, context)
        
        if 'tasks_ids' in full_data or 'attachments' in full_data:
            self.write(cr,uid,[cur_id], {'tasks_ids':full_data['tasks_ids'] or [], 'attachments':full_data['attachments'] or []}, context=context)
                
        #add the form creation to the opportunity history
        if 'opportunity_id' in data:
            text = _("Created a new CPI form")
            oppManager = self.pool.get('crm.lead')
            oppManager.message_append(cr, uid, [data['opportunity_id']], text, body_text=text, context=context)
        
        return cur_id

    def write(self, cr, uid, ids, vals, context=None):
        if 'tasks_ids' in vals:
            for cur_task_entry in vals['tasks_ids']:
                if cur_task_entry[0] == 0 and cur_task_entry[1] == 0:
                    cur_task_entry[2]['apem_subform_id'] = 'crm.form.cpi,'+str(ids[0])
                    cur_task_entry[2]['project_id'] = self._get_project_id(cr, uid)
                    cur_task_entry[2]['group_stage'] = self.get_state_from_user(cr, uid, context=context)

        res = super(apem_cpi_form, self).write(cr, uid, ids, vals, context)
        
        #link all the attachments of the list to the current form
        if 'attachments' in vals:
            #vals['attachment'] is of the form [[6, False, [1,2,3]]] since it's a many2many
            #where 1st => "unlink all + recreate links"
            #      2nd => False
            #      3rd => list of the ir.attachments ids to link
            if len(vals['attachments'])>0 and vals['attachments'][0][0] == 6:
                to_write = {'res_id':ids[0], 'res_model':'crm.form.cpi', 'res_name':self.browse(cr,uid,ids[0],context).name}
                self.pool.get('ir.attachment').write(cr,uid, vals['attachments'][0][2], to_write, context=context)
        
        for cur_id in ids:
            cur_cpi = self.browse(cr,uid,cur_id,context)
                    
            #add the form validation to the opportunity history
            if 'decision' in vals and 'opportunity_id' in cur_cpi:
                text = _("Answered to the CPI %s") % cur_cpi.name
                oppManager = self.pool.get('crm.lead')
                oppManager.message_append(cr, uid, [cur_cpi['opportunity_id']], text, body_text=text, context=context)
                
        return res
    
    def create_new_version(self, cr, uid, ids, context=None):
        default={}
        default["attachments"] = []
        default["tasks_ids"] = []
        default['children_forms'] = []
        
        default["answer"] = False
        default["decision"] = False
        default["name"] = self._defaults['name'](self,cr,uid,context=context)
        
        for cur_id in ids:
            default["parent_form"] = cur_id
            dup_form = super(apem_cpi_form,self).copy(cr, uid, cur_id, default=default, context=context)
            
        '''Redirects to the last form created'''
        return {
            'name':_("CPI form created"),
            'view_mode': 'form',
            'view_type': 'form',
            'res_id' : dup_form,
            'res_model': 'crm.form.cpi',
            'type': 'ir.actions.act_window'
        }
        
    def copy(self, cr, uid, id, default=None, context=None):
        return self.create_new_version(cr, uid, [id], context=context)['res_id']

    def get_state_assignee(self, cr, uid, ids, group, context=None):
        '''This function should only be called with a single id. list isn't handled'''
        forms = self.pool.get('crm.form.cpi').browse(cr,uid,ids,context=context)
        res = {}
        
        cur_form = forms[0]
        assignee = {}
        
        #build "switch-case" dictionary
        gr = self._get_PM_group(cr, uid, context=context)
        if len(gr) == 1:
            assignee[gr[0]] = cur_form.in_charge_PM.id
        gr = self._get_ENG_group(cr, uid, context=context)
        if len(gr) == 1:
            assignee[gr[0]] = cur_form.in_charge_Engineering.id
        gr = self._get_METH_group(cr, uid, context=context)
        if len(gr) == 1:
            assignee[gr[0]] = cur_form.in_charge_Methods.id
        gr = self._get_SALES_group(cr, uid, context=context)
        if len(gr) == 1:
            assignee[gr[0]] = cur_form.in_charge_Salesman.id
            
        if group.id in assignee:
            if assignee[group.id] != False:
                return assignee[group.id]
            else:
                raise osv.except_osv(_('Error:'), _('This form doesn\'t have anyone assigned for this stage.'))
        raise osv.except_osv(_('Error:'), _('This form doesn\'t have a step corresponding to the group "%s".')%group.name)
    
    def get_state_from_user(self, cr, uid, context=None):
        groups = self.pool.get('res.users').browse(cr,uid,uid, context=context).groups_id
        gr_id = None
        
        cur_id = (self._get_SALES_group(cr, uid, context=context))[0]
        if cur_id in [g.id for g in groups]:
            gr_id = cur_id
        
        cur_id = (self._get_METH_group(cr, uid, context=context))[0]
        if cur_id in [g.id for g in groups]:
            gr_id = cur_id
        
        cur_id = (self._get_ENG_group(cr, uid, context=context))[0]
        if cur_id in [g.id for g in groups]:
            gr_id = cur_id
        
        cur_id = (self._get_PM_group(cr, uid, context=context))[0]
        if cur_id in [g.id for g in groups]:
            gr_id = cur_id
            
        if gr_id:
            states = self.pool.get('apem.task.type').search(cr, uid, [('group_id', '=', gr_id)], context=context)
            if len(states) == 1:
                return states[0]
        
        return False
        
apem_cpi_form()