# -*- coding: utf-8 -*-

from osv import fields, osv
import time
from tools.translate import _

class sales_price(osv.osv):
    _name="crm.form.sales.price"
    _description="Price gap analysis"
    _columns={
              'main_form': fields.many2one('crm.form.price', "Main form"),
              'quantity': fields.integer('Quantity'),
              'regular_price': fields.float('Regular price', digits=(10,4)),
              'competitor_price': fields.float('Competitor price', digits=(10,4)),
              'wished_price': fields.float('Wished price', digits=(10,4)),
              
              'accepted_price': fields.float('Accepted price', digits=(10,4), write=['base.group_sale_manager'], read=['base.group_sale_salesman']),
              
              'margin': fields.float('Margin %', digits=(10,4), write=['base.group_sale_manager'], read=['base.group_sale_manager']),
              'direct_cost': fields.float('Direct cost', digits=(10,4), write=['base.group_sale_manager'], read=['base.group_sale_manager'])

             }
    
    def onchange_margin(self, cr, uid, ids, margin, direct_cost):
        margin = float(margin)
        direct_cost = float(direct_cost)
        if margin >= 100.0:
            return {'warning':{
                        'title': _('Warning!'),
                        'message': _('Margin must be less than 100%')}}
        return {'value':{'accepted_price':(direct_cost/(1.0-(margin/100.0)))}}
        
    def onchange_accepted_price(self, cr, uid, ids, accepted_price, direct_cost):
        accepted_price = float(accepted_price)
        direct_cost = float(direct_cost)
        return {'value':{'margin':((1.0-(direct_cost/accepted_price))*100.0)}}
    
    def fields_get(self, cr, uid, fields=None, context=None):
        """Sets the field 'accepted_price' as readonly if the user doesn't have the right to write on it"""
        user = self.pool.get('res.users').browse(cr, uid, uid)
        res = super(sales_price,self).fields_get(cr, uid, fields, context)
        if 'accepted_price' in res:
            res['accepted_price']['readonly'] = True
            for cur_group in user.groups_id:
                if cur_group.full_name == 'Sales Management / Manager':
                    del res['accepted_price']['readonly']
            
        return res
    
sales_price()    

class apem_price_form(osv.osv):
    _name="crm.form.price"
    _inherits={"apem.form.abstract" : "abstract_form_id"}
    _description="Form FC00062A for a special price demand"
    
    def _get_SALES_group(self, cr, uid, context=None):
        categ = self.pool.get('ir.module.category').search(cr, uid, [('name', '=', 'Sales Management')], context=context)
        groups = self.pool.get('res.groups').search(cr, uid, [('name', '=', 'User - All Leads'), ('category_id', "in", categ)], context=context)
        return groups
    
    def _get_PM_group(self, cr, uid, context=None):
        categ = self.pool.get('ir.module.category').search(cr, uid, [('name', '=', 'Sales Management')], context=context)
        groups = self.pool.get('res.groups').search(cr, uid, [('name', '=', 'Manager'), ('category_id', "in", categ)], context=context)
        return groups
        
    def _get_PM_members(self, cr, uid, context=None):
        groups = self._get_PM_group(cr, uid, context=context)
        if len(groups) == 1:
            return [(p.id, p.name) for p in self.pool.get('res.groups').browse(cr,uid,groups[0]).users]
        return []
    
    _columns={
              #link
              'sales_orders': fields.many2many('sale.order', 'res_so_price_rel', 'price_id', 'so_id', 'Ordered in'),              
              'competitors': fields.related('abstract_form_id', 'opportunity_id', 'competitors', type="many2many", relation="res.partner", string="Competitors", store=False, readonly=True),
              
              'demand_date': fields.date('Demand date', readonly=True),
              'demand_reason': fields.text('Reason for the demand'),         
              'requester_comment': fields.text('Requester comment'),
              
              'series': fields.many2one('product.series','Series'),
              'part_number': fields.char('Part number',size=64),
              'potential': fields.integer('Annual potential', help="quantity/year"),              
              'sales_price': fields.one2many('crm.form.sales.price', 'main_form', 'Sales price'),
              
              'visa': fields.text('Product Manager Visa',write=['base.group_sale_manager'], read=['base.group_sale_salesman']),
              
              #workflow
              'in_charge_PM': fields.many2one('res.users', 'PM in charge', help="Product manager in charge of this demand",
                                              selection=_get_PM_members),
              
              #hierarchy
              'parent_form': fields.many2one('crm.form.price', 'Duplicated from', readonly=True),
              'children_forms': fields.one2many('crm.form.price', 'parent_form', 'Children', readonly=True),
              }
    _defaults={
              'name':  lambda self,cr,uid,context={}: self.pool.get('ir.sequence').get(cr, uid, 'crm.form.price'), 
              'demand_date': lambda *a: time.strftime('%Y-%m-%d'),
              }
    
    def _get_project_id(self, cr, uid):
        projectManager = self.pool.get('project.project')
        res = projectManager.search(cr,uid,[('name', '=', 'Price forms')])
        if len(res) == 1:
            return res[0]
        else:
            return False
    
    def fields_get(self, cr, uid, fields=None, context=None):
        """Sets the field 'visa' as readonly if the user doesn't have the right to write on it"""
        user = self.pool.get('res.users').browse(cr, uid, uid)
        res = super(apem_price_form,self).fields_get(cr, uid, fields, context)
        if 'visa' in res:
            res['visa']['readonly'] = True
            for cur_group in user.groups_id:
                if cur_group.full_name == 'Sales Management / Manager':
                    del res['visa']['readonly']
            
        if 'in_charge_PM' in res:
            res['in_charge_PM']['domain'] = "[('id','in',["+",".join([str(m[0]) for m in self._get_PM_members(cr,uid)]) + "])]"
        
        return res
    
    def create(self, cr, uid, data, context=None):
        '''tasks_ids need some special values injection based on the form id.
           To achieve this, we remove them from the initial creation and call a "write" on the created form afterwards'''
        
        full_data = data.copy()
        if 'tasks_ids' in data:
            del data['tasks_ids']
            
        cur_id = super(apem_price_form, self).create(cr, uid, data, context)
        
        if 'tasks_ids' in full_data:
            self.write(cr,uid,[cur_id], {'tasks_ids':full_data['tasks_ids']}, context=context)
                
        #add the form creation to the opportunity history
        if 'opportunity_id' in data:
            text = _("Created a new Price form")
            oppManager = self.pool.get('crm.lead')
            oppManager.message_append(cr, uid, [data['opportunity_id']], text, body_text=text, context=context)
        
        return cur_id

    def write(self, cr, uid, ids, vals, context=None):
        if 'tasks_ids' in vals:
            for cur_task_entry in vals['tasks_ids']:
                if cur_task_entry[0] == 0 and cur_task_entry[1] == 0:
                    cur_task_entry[2]['apem_subform_id'] = 'crm.form.price,'+str(ids[0])
                    cur_task_entry[2]['project_id'] = self._get_project_id(cr, uid)
                    cur_task_entry[2]['group_stage'] = self.get_state_from_user(cr, uid, context=context)
   
        res = super(apem_price_form, self).write(cr, uid, ids, vals, context)
        
        for cur_id in ids:
            cur_price = self.browse(cr,uid,cur_id,context)
                    
            #add the form validation to the opportunity history
            if 'visa' in vals and 'opportunity_id' in cur_price:
                text = _("Answered to the special price request %s") % cur_price.name
                oppManager = self.pool.get('crm.lead')
                oppManager.message_append(cr, uid, [cur_price['opportunity_id']], text, body_text=text, context=context)
                
        return res
    
    def create_new_version(self, cr, uid, ids, context=None):
        default={}
        default["sales_price"] = []
        default["tasks_ids"] = []
        default['children_forms'] = []
        
        default["visa"] = False
        default["name"] = self._defaults['name'](self,cr,uid,context=context)
        
        for cur_id in ids:
            default["parent_form"] = cur_id
            dup_form = super(apem_price_form,self).copy(cr, uid, cur_id, default=default, context=context)
            
        '''Redirects to the last form created'''
        return {
            'name':_("Price form created"),
            'view_mode': 'form',
            'view_type': 'form',
            'res_id' : dup_form,
            'res_model': 'crm.form.price',
            'type': 'ir.actions.act_window'
        }
        
    def copy(self, cr, uid, id, default=None, context=None):
        return self.create_new_version(cr, uid, [id], context=context)['res_id']
        
    def get_state_assignee(self, cr, uid, ids, group, context=None):
        '''This function should only be called with a single id. list isn't handled'''
        forms = self.pool.get('crm.form.price').browse(cr,uid,ids,context=context)
        res = {}
        
        cur_form = forms[0]
        assignee = {}
        
        #build "switch-case" dictionary
        gr = self._get_PM_group(cr, uid, context=context)
        if len(gr) == 1:
            assignee[gr[0]] = cur_form.in_charge_PM.id
        gr = self._get_SALES_group(cr, uid, context=context)
        if len(gr) == 1:
            assignee[gr[0]] = cur_form.in_charge_Salesman.id

        if group.id in assignee:
            if assignee[group.id] != False:
                return assignee[group.id]
            else:
                raise osv.except_osv(_('Error:'), _('This form doesn\'t have anyone assigned for this stage.'))
        raise osv.except_osv(_('Error:'), _('This form doesn\'t have a step corresponding to the group "%s".')%group.name)
    
    def get_state_from_user(self, cr, uid, context=None):
        groups = self.pool.get('res.users').browse(cr,uid,uid, context=context).groups_id
        gr_id = None
        
        cur_id = (self._get_SALES_group(cr, uid, context=context))[0]
        if cur_id in [g.id for g in groups]:
            gr_id = cur_id
        
        cur_id = (self._get_PM_group(cr, uid, context=context))[0]
        if cur_id in [g.id for g in groups]:
            gr_id = cur_id
            
        if gr_id:
            states = self.pool.get('apem.task.type').search(cr, uid, [('group_id', '=', gr_id)], context=context)
            if len(states) == 1:
                return states[0]
        
        return False
    
apem_price_form()