# -*- coding: utf-8 -*-

from osv import fields, osv
from tools.translate import _
    
class apem_task_type(osv.osv):
    _name = "apem.task.type"
    _description = 'Apem groups as task stages'
    _order = 'sequence'
    
    _columns = {
        'name': fields.char ('Name', size=128, required=True, translate=True),
        'sequence': fields.integer('Sequence'),
        'group_id': fields.many2one('res.groups', 'Group' ),
        'project_ids': fields.many2many('project.project', 'apem_task_type_rel', 'type_id', 'project_id', 'Projects', invisible="True"),
                }
    _defaults = {
        'sequence': 1
                }        
    
apem_task_type()

class apem_task(osv.osv):
    _inherit = 'project.task'
    
    def _get_group_stages(self, cr, uid, ids, domain, read_group_order=None, access_rights_uid=None, context=None):
        stage_obj = self.pool.get('apem.task.type')
        project_id = self._resolve_project_id_from_context(cr, uid, context=context)
        order = stage_obj._order
        access_rights_uid = access_rights_uid or uid
        if read_group_order == 'group_stage desc':
            # lame way to allow reverting search, should just work in the trivial case
            order = '%s desc' % order
        if project_id:
            domain = ['|', ('id','in',ids), ('project_ids','in',project_id)]
        else:
            domain = []
            
        stage_ids = stage_obj._search(cr, uid, domain, order=order, access_rights_uid=access_rights_uid, context=context)
        result = stage_obj.name_get(cr, access_rights_uid, stage_ids, context=context)
        # restore order of the search
        result.sort(lambda x,y: cmp(stage_ids.index(x[0]), stage_ids.index(y[0])))
        return result
    
    _group_by_full = {
        'group_stage': _get_group_stages
                    }
    
    _columns = {
            'apem_form_id': fields.many2one('apem.form.abstract', 'Corresponding abstract form', readonly=True),
            'apem_opportunity_id': fields.related('apem_form_id', 'opportunity_id', type="many2one", \
                                                relation="crm.lead", string="Corresponding opportunity", \
                                                context="{'form_view_ref':'crm.crm_case_form_view_oppor'}", \
                                                store=False, readonly=True),
            'apem_subform_id': fields.reference('Corresponding form',[
                                ('crm.form.cpi','CPI form'),
                                ('crm.form.price','Price form')], 128, readonly=True),
                
            'stage_work_done': fields.boolean('Work filled in', invisible=True),
            'group_stage': fields.many2one('apem.task.type', 'Groups involved'), #TODO: add domain through "fields_view_get" and "_get_group_stages"
            
            'user_avatar': fields.related('user_id', 'avatar', type='binary', string='User Avatar', readonly=True),
                }
    _defaults = {
            'stage_work_done': lambda *a: False
                }
    
    def _is_in_assignee_group(self, cr, uid, ids, context=None):
        ''' Form tasks are passed from one group to another to get the work done.
            Even if there's an assignee, everyone from the same group should be able to update the task in case the assignee is sick.
            This function ensure the user modifying the task belongs to the right group'''
        for cur_task in self.browse(cr,uid,ids,context=context):
            # if no group stage, we assume it's a normal task and not a form one. we don't handle rights for the former.
            if 'group_stage' in cur_task:
                group = cur_task['group_stage']['group_id']
                if not uid in [u.id for u in group.users]:
                    return False
        
        return True
    
    def _apem_task_send_salesman_email(self, cr, uid, task_id, context=None):        
        mail_template_manager = self.pool.get('email.template')
        cur_task = self.browse(cr, uid, task_id, context=context)
        
        template_id = mail_template_manager.search(cr,uid,[("name", "=", "task state changed")])[0]
        web_root_url = self.pool.get('ir.config_parameter').get_param(cr, uid, 'web.base.url')
        cur_url = ("%s#view_type=page&model=project.task&id=%d")%(web_root_url,task_id)
        return mail_template_manager.send_mail(cr, uid,template_id,task_id,context=dict(context.items() + {'task_url':cur_url}.items()))
    
    def _apem_task_send_email(self, cr, uid, task_id, context=None):        
        mail_template_manager = self.pool.get('email.template')
        cur_task = self.browse(cr, uid, task_id, context=context)
        salesman = cur_task.apem_opportunity_id.user_id
        
        template_id = mail_template_manager.search(cr,uid,[("name", "=", "task assigned")])[0]
        web_root_url = self.pool.get('ir.config_parameter').get_param(cr, uid, 'web.base.url')
        cur_url = ("%s#view_type=page&model=project.task&id=%d")%(web_root_url,task_id)
        return mail_template_manager.send_mail(cr, uid,template_id,task_id,context=dict(context.items() + {'task_url':cur_url}.items()))
    
    def _apem_task_close_send_email(self, cr, uid, task_id, context=None):        
        mail_template_manager = self.pool.get('email.template')
        cur_task = self.browse(cr, uid, task_id, context=context)
        
        template_id = mail_template_manager.search(cr,uid,[("name", "=", "task closed")])[0]
        web_root_url = self.pool.get('ir.config_parameter').get_param(cr, uid, 'web.base.url')
        cur_url = ("%s#view_type=page&model=project.task&id=%d")%(web_root_url,task_id)
        return mail_template_manager.send_mail(cr, uid,template_id,task_id,context=dict(context.items() + {'task_url':cur_url}.items()))
    
    def _apem_task_state_changed(self, cr, uid, ids, group_stage_id, context=None):
        for cur_id in ids:
            cur_task = self.browse(cr, uid, cur_id)
            
            if cur_task.stage_work_done == False:
                raise osv.except_osv(_('Error:'), _('Please fill-in the time you spent on the task "%s".')%cur_task.name)
            else:
                group = self.pool.get('apem.task.type').browse(cr,uid,group_stage_id,context=context)
                assignee = cur_task.apem_subform_id.get_state_assignee(group['group_id'], context=context)
                
                cur_task.write({'stage_work_done': False, 'user_id':assignee}, context=context)
                
                #email the salesman
                email_sent = self._apem_task_send_email(cr, uid, cur_id, context=context)
                if 'apem_opportunity_id' in cur_task:
                    opp_salesman = cur_task.apem_opportunity_id.user_id.id
                    if opp_salesman != assignee:
                        email_sent = self._apem_task_send_salesman_email(cr, uid, cur_id, context=context)
                
                #Add message to opportunity
                opp_id = cur_task['apem_opportunity_id']
                text = _("Reassigned a task to %s") % self.pool.get('res.users').browse(cr,uid,assignee, context=context).name
                oppManager = self.pool.get('crm.lead')
                oppManager.message_append(cr, uid, [opp_id], text, body_text=text, context=context)
                
        return True

    def write(self, cr, uid, ids, vals, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]
            
        #everyone can modify sequences. Otherwise you need to be in the right group
        if self._is_in_assignee_group(cr, uid, ids,context=context) or (len(vals) == 1 and 'sequence' in vals):
            if 'group_stage' in vals:
                self._apem_task_state_changed(cr, uid, ids, vals['group_stage'], context=context)
                
            r = super(apem_task, self).write(cr, uid, ids, vals, context=context)            
            return r
        else:
            raise osv.except_osv(_('Error:'), _('You don\'t have the right to update this record'))
        
    def action_close(self, cr, uid, ids, context=None):
        for cur_task in self.browse(cr,uid,ids,context=context):
            if 'group_stage' in cur_task:
                self._apem_task_close_send_email(cr, uid, cur_task.id, context=context)
        
        return super(apem_task,self).action_close(cr, uid, ids, context=context)

apem_task()


class apem_project(osv.osv):
    _inherit = 'project.project'
    
    _columns = {
                'apem_types_ids': fields.many2many('apem.task.type', 'apem_task_type_rel', 'project_id', 'type_id', 'Apem tasks stages')
                }
apem_project()