# -*- coding: utf-8 -*-

from osv import osv, fields
from mail.mail_message import to_email
import time

class ugly_hack(osv.osv):
    """
    mail_thread.message_capable_models does a terrible job at detecting message capable classes with more than 1 lvl inheritance
    that class adds crm.lead to the list even if there's another level
    """
    _inherit = "mail.thread"
    
    def message_capable_models(self, cr, uid, context=None):
        res = super(ugly_hack,self).message_capable_models(cr, uid, context)
        
        #force leads into the results
        model_name = "crm.lead"
        model = self.pool.get(model_name)
        res[model_name] = model._description
        
        #force partner address into the results
        model_name = "res.partner.address"
        model = self.pool.get(model_name)
        res[model_name] = model._description
        
        # del the mail.thread that this class adds
        del res["mail.thread"]
        
        return res
        

class apem_lead(osv.osv):
    _inherit = "crm.lead"
    _description = "Lead/Opportunity"
    
    _columns = {
                #import compatibility
                '_GM_id': fields.char('Goldmine ID',size=128),
                
                #apem internal organization
                'sales_assistant': fields.many2one('res.users', 'Sales assistant', states={'draft': [('readonly', False)]}, select=True),
                'customer_service_contact': fields.many2one('res.users', 'Customer service contact', states={'draft': [('readonly', False)]}, select=True),
                
                #client details
                'partner_activity': fields.many2one('res.partner.activity', "Partner activity"),
                'partner_market': fields.many2one('res.partner.market', "Partner market"),
                
                #competition
                'won_lost_to': fields.many2one('res.partner', "Won/Lost to"),
                'won_lost_reason': fields.many2one('crm.lead.wonlostreason', "Won/Lost reason"),
                'final_client': fields.many2one('res.partner', "Final Client"),
                'competitors': fields.many2many('res.partner', 'res_lead_competitors_rel', 'competitor_id', 'competitor_lead_id', 'Competitors'),
                
                #NAO
                'apem_opportunity':  fields.boolean('APEM Opportunity'),
                'key_account': fields.related('partner_id', 'key_account', type="boolean", relation="res.partner", string="Key account", store=False, readonly=True),
                
                #for kanban display
                'user_avatar': fields.related('user_id', 'avatar', type='binary', string='User Avatar', readonly=True)
                }
    
    def write(self, cr, uid, ids, vals, context=None):
        """if partner has been changed, updates the sales team and salesman accordingly"""
        if 'partner_id' in vals and vals['partner_id']:
            partner = self.pool.get('res.partner').browse(cr, uid, vals['partner_id'])
            vals['user_id'] = partner.user_id.id
            vals['section_id'] = partner.section_id.id
            
        return super(apem_lead, self).write(cr, uid, ids, vals, context)
    
    def onchange_section_id(self, cr, uid, ids, section_id):
        res = {"domain":{"user_id":False,"sales_assistant":False,"customer_service_contact":False}}
        
        if section_id:
            section = self.pool.get('crm.case.section').browse(cr, uid, section_id)
            
            section_domain = "[('id','in', ["+','.join(map(str,[r.id for r in section.member_ids]))+"])]" or "[]"
            res['domain']['user_id'] = section_domain
            res['domain']['sales_assistant'] = section_domain
            res['domain']['customer_service_contact'] = section_domain
        
        return res
    
    def onchange_partner_id(self, cr, uid, ids, partner_id, email_from):
        res = super(apem_lead,self).onchange_partner_id(cr, uid, ids, partner_id, email_from)
                
        if partner_id:        
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
    
            if partner.section_id:
                res['value'] = res.get('value',{})
                res['value']['section_id'] = partner.section_id.id
                sect_res = self.onchange_section_id(cr, uid, ids, partner.section_id.id)
                res['domain'] = res.get('domain',{})
                res['value'] = res.get('value',{})
                res['warning'] = res.get('warning',{})
                res['domain'].update(sect_res.get('domain',{}))
                res['value'].update(sect_res.get('value',{}))
                res['warning'].update(sect_res.get('warning',{}))
                
                vals = { "user_id": partner.user_id.id or False,
                         "sales_assistant": partner.sales_assistant.id or False,
                         "customer_service_contact": partner.customer_service_contact.id or False}
                res['value'].update(vals)
    
        return res

#    def onchange_partner_id(self, cr, uid, ids, add, email=False):
#        """
#        This function returns value of sales team and its members
#        """
#        if not add:
#            my_res =  {'value': {'sales_assistant': False, 'customer_service_contact': False, \
#                              'user_id': False, 'section_id': False, 'partner_market':False}}
#        else:
#            partner = self.pool.get('res.partner').browse(cr, uid, add)
#            my_res = {'value': {'sales_assistant': partner.sales_assistant.id,\
#                               'customer_service_contact': partner.customer_service_contact.id, \
#                               'user_id': partner.user_id.id, 'section_id': partner.section_id.id, \
#                               'partner_market':partner.market.id}}
#            
#        result =  super(apem_lead, self).onchange_partner_id(cr, uid, ids, add)
#        return dict(list(result.items()) + list(my_res.items()));

    
    def _lead_create_partner(self, cr, uid, lead, context=None):
        """Modified to add a few attributes to the created partner"""
        partner = self.pool.get('res.partner')
        partner_id = partner.create(cr, uid, {
                    'name': lead.partner_name or lead.contact_name or lead.name,
                    'user_id': lead.user_id.id,
                    'comment': lead.description,
                    'section_id': lead.section_id.id or False,
                    'address': [],
                    'activity': lead.partner_activity.id,
                    'market': lead.partner_market.id,
                    'source': lead.channel_id.id
        })
        return partner_id
    
    def convert_partner(self, cr, uid, ids, action='create', partner_id=False, context=None):
        """
        This function convert partner based on action.
        if action is 'create', create new partner with contact and assign lead to new partner_id.
        otherwise assign lead to specified partner_id
        Modified to call the _lead_create_partner_address for all the actions and not only  create
        """
        if context is None:
            context = {}
        partner_ids = {}
        for lead in self.browse(cr, uid, ids, context=context):
            if action == 'create':
                if not partner_id:
                    partner_id = self._lead_create_partner(cr, uid, lead, context=context)
            """LP: Changed the self._lead_create_partner_address to happen for all the actions
                    it must also be called AFTER _lead_set_partner since that function sets the contact to a default one"""
            self._lead_set_partner(cr, uid, lead, partner_id, context=context)
            partner_ids[lead.id] = partner_id
            self._lead_create_partner_address(cr, uid, lead, partner_id, context=context)
        return partner_ids
    
    def _lead_create_partner_address(self, cr, uid, lead, partner_id, context=None):
        """Modified to create the contact only if a contact with the same email doesn't already exists"""
        address = self.pool.get('res.partner.address')
        contact_email = lead.email_from and to_email(lead.email_from)[0]
        same_contacts = address.search(cr, uid, [('email', '=', contact_email), ('partner_id', '=', partner_id)], context=context)
        
        if len(same_contacts) == 1:
            new_contact = same_contacts[0]
        else:
            new_contact = address.create(cr, uid, {
                    'partner_id': partner_id,
                    'name': lead.contact_name,
                    'phone': lead.phone,
                    'mobile': lead.mobile,
                    'email': lead.email_from and to_email(lead.email_from)[0],
                    'fax': lead.fax,
                    'title': lead.title and lead.title.id or False,
                    'function': lead.function,
                    'street': lead.street,
                    'street2': lead.street2,
                    'zip': lead.zip,
                    'city': lead.city,
                    'country_id': lead.country_id and lead.country_id.id or False,
                    'state_id': lead.state_id and lead.state_id.id or False,
                })
        self.write(cr, uid, [lead.id], {'partner_address_id':new_contact}, context=context)
        lead.partner_address_id = address.browse(cr, uid, [lead.id], context=context)[0]
        return new_contact
    
    def _convert_opportunity_data(self, cr, uid, lead, customer, section_id=False, context=None):
        """Modified to return the actual contact_id. The old function only returned the default one for the given partner"""
        crm_stage = self.pool.get('crm.case.stage')
        contact_id = False
        if customer:
            contact_id = (lead.partner_address_id and lead.partner_address_id.id) or self.pool.get('res.partner').address_get(cr, uid, [customer.id])['default']
        if not section_id:
            section_id = lead.section_id and lead.section_id.id or False
        if section_id:
            stage_ids = crm_stage.search(cr, uid, [('sequence','>=',1), ('section_ids','=', section_id)])
        else:
            stage_ids = crm_stage.search(cr, uid, [('sequence','>=',1)])
        stage_id = stage_ids and stage_ids[0] or False
        return {
                'planned_revenue': lead.planned_revenue,
                'probability': lead.probability,
                'name': lead.name,
                'partner_id': customer and customer.id or False,
                'user_id': (lead.user_id and lead.user_id.id),
                'type': 'opportunity',
                'stage_id': stage_id or False,
                'date_action': time.strftime('%Y-%m-%d %H:%M:%S'),
                'date_open': time.strftime('%Y-%m-%d %H:%M:%S'),
                'partner_address_id': contact_id,
        }
    
apem_lead()