# -*- coding: utf-8 -*-

from osv import osv, fields

class apem_partners(osv.osv):
    """ This module adapts the res.partner.contact module to APEM needs """
    _inherit = "res.partner"
    
    def _get_related_partner_reverse(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for cur_entry in ids:
            res[cur_entry] = self.search(cr,uid,[('related_partners_ids.id','in',[cur_entry])],context=context)

        return res
    
    def _search_related_partner_reverse(self, cr, uid, obj, name, args, context):
        domain = []
        for cur_dom in args:
            if cur_dom[0] == name:
                targets = self.browse(cr,uid,cur_dom[2],context=context)
                #@awesome: 1-line 2level object flattening
                rel_partners = [i.id for cur_target in targets for i in cur_target['related_partners_ids']]
                
                domain.append(('id','in',rel_partners))
            else:
                domain.append(cur_dom)
        
        return domain

    _columns = {
              'activity': fields.many2one('res.partner.activity', "Activity"),
              'market': fields.many2one('res.partner.market', "Market"),
              'source': fields.many2one('crm.case.channel', 'First contact source', help="First communication source (mail, direct, phone, ...)"),
              
              'hidden_salesteam_members': fields.related('section_id', 'member_ids', type="many2many", relation="res.users", string="Sales Team Members", store=False),
              #previous related field and following domains are part of some ugly trick to get OERP6.1 to understand domain restrictions from relation attributes
              'sales_assistant': fields.many2one('res.users', 'Sales assistant', domain="[('id','in', [id for id in hidden_salesteam_members[0][2]])]"),
              'customer_service_contact': fields.many2one('res.users', 'Customer service contact', domain="[('id','in',[id for id in hidden_salesteam_members[0][2]])]"),
              'user_id': fields.many2one('res.users', 'Salesman', domain="['&',('hidden_salesteam_members','!=',False),('id','in',[id for id in hidden_salesteam_members[0][2]])]",
                                         help='The internal user that is in charge of communicating with this partner if any.'),
                
              'key_account': fields.boolean('Key Account'),
              'retrocessed_to': fields.many2many('res.partner', 'apem_retrocessed_partners_rel', 'p1_id', 'p2_id', 'Retrocessed to',
                                                       help='List of distributors who should handle this client instead of us'),
              
              'related_partners_ids': fields.many2many('res.partner', 'apm_related_partners_rel', 'p1_id', 'p2_id', 'Related partners',
                                                       help='In sales orders, contacts can be chosen from the related companies as well'),
              'related_partner_reverse_ids': fields.function(_get_related_partner_reverse, None, fnct_search=_search_related_partner_reverse,
                                                method=True, type="many2many" ,store=False, readonly=True),
                }
    
    def onchange_section_id(self, cr, uid, ids, add):
        if not add:
            myRes =  {}
        else:
            section = self.pool.get('crm.case.section').browse(cr, uid, add)
            myRes = {'domain': {'sales_assistant': "[('id','in', ["+','.join(map(str,[res.id for res in section.member_ids]))+"])]",
                                'customer_service_contact': "[('id','in', ["+','.join(map(str,[res.id for res in section.member_ids]))+"])]",
                                'user_id': "[('id','in', ["+','.join(map(str,[res.id for res in section.member_ids]))+"])]"}}
        
        return myRes        
    
apem_partners()