# -*- coding: utf-8 -*-

from osv import osv,fields

class apem_pricelist_breakpoints(osv.osv):
    _name = "apem.pricelist.breakpoints"
    
    _columns = {
                'pricelist_id': fields.many2one('product.pricelist', 'Pricelist', store=False),
                'product_ids': fields.many2many('product.product', 'pricelist_breakpoints_products_rel',
                                                'prod_id', 'prices_bp_id', 'Product', store=False),
                'prices': fields.text('Prices', readonly=True, store=False)
                }
        
    def _get_pricelist_breakpoints(self, cr, uid, pricelist_id, product_id, context=None):
        pricelist_items_manager = self.pool.get('product.pricelist.item')
        items_ids = pricelist_items_manager.search(cr,uid,[('price_version_id.pricelist_id','=',pricelist_id),('product_id','=',product_id)],
                                                   order='min_quantity',context=context)
        
        res = [i.min_quantity for i in pricelist_items_manager.browse(cr,uid,items_ids,context=context)]
        if 0 not in res:
            res.insert(0,0)
        return res
        #return [1,50,1000]
    
    def compute_prices(self, cr, uid, ids, pricelist_id, product_ids, context=None):
        if not pricelist_id:
            return {'value': {'prices': ''}}
        
        prods = []
        for p in product_ids:
            if p[0] == 4:
                prods.append(p[1])
            elif p[0] == 6:
                prods = prods + p[2]
                
        if not len(prods) > 0:
            return {'value': {'prices': ''}}
        
        pricelist = self.pool.get('product.pricelist').browse(cr,uid,pricelist_id)
        res = ""
       
        for cur_prod in prods:
            qty = self._get_pricelist_breakpoints(cr, uid, pricelist_id, cur_prod, context=context)
            if not qty:
                qty = [1]
            
            prod = self.pool.get('product.product').browse(cr, uid, cur_prod)
            res = ("%s\n%s:\n") % (res,prod.default_code)
            
            for q in qty:
                price_dict = pricelist.price_get(cur_prod, q, context=context)
            
                #from pricelist
                if price_dict[pricelist_id]:
                    price = price_dict[pricelist_id]
                #from product
                else:
                    price = prod.list_price

                res = ("%s%d  =>  %.4f %s:\n") % (res,q,price,pricelist.currency_id.name)
               
        return {'value': {'prices': res}}
         
    
    def create(*args):
        return False
    
    def write(*args):
        return False
    
    def unlink(*args):
        return False
    
apem_pricelist_breakpoints()