# -*- coding: utf-8 -*-

from osv import osv, fields
from tools.translate import _
import decimal_precision as dp

class apem_products(osv.osv):
    """ This module adapts the product.product module to APEM needs """
    _inherit = "product.product"

    _columns = {
                #override to add rights
                'default_code' : fields.char('Reference', size=64, select=True,
                                             write=['stock.group_stock_manager']),
        
                'family': fields.many2one('product.families', "Family"),
                'series': fields.many2one('product.series', "Series"),
                'moq': fields.float("Minimal order quantity"),
                
                'designation': fields.char('Designation', size=64,
                                help="If the designation is the same as the name, \
                                        the name will be displayed between brackets instead of the code. \
                                        Otherwise the designation must be equal to the code.")
                }

    def check_packaging_available(self, cr, uid, to_get, pack_quantities, context=None):
        '''
            Due to performance issues, this function doesn't check for combination of packages.
        '''
        for q in pack_quantities:
            if to_get % q == 0:
                return True
                        
        return False    
    
    def on_change_moq(self, cr, uid, id, moq, packaging_ids, context=None):
        res = {}
        pack_quantities = []
        
        for p in packaging_ids:
            if p[2] and 'qty' in p[2]:
                pack_quantities.append(p[2]['qty'])
            elif p[1] != False:
                pack_items = self.pool.get("product.packaging").browse(cr,uid,p[1], context=context)
                pack_quantities.append(pack_items['qty'])
        
        if not self.check_packaging_available(cr, uid, moq, pack_quantities, context=context):
            res = { 'warning' : {
                        'title': _('Warning!'),
                        'message': _('The quantity you selected can\'t be obtained with currently available packagings')}
            }
            
        return res

    def name_get(self, cr, user, ids, context=None):
        old_res = super(apem_products,self).name_get(cr, user, ids, context=context)
        new_res = []
        
        for i in old_res:
            cur_prod = self.browse(cr,user,i[0],context=context)
            name = cur_prod.name or ''
            code = cur_prod.default_code or False
            desig = cur_prod.designation or False
            if desig:
                if desig == name:
                    i = (i[0],i[1].replace('[%s] %s' % (code,name), '[%s] %s' % (name,code)))
            new_res.append(i)
        return new_res
    
apem_products()

class product_template(osv.osv):
    _inherit = "product.template"
    
    _columns = {
                #override to add rights
                'standard_price': fields.float('Cost Price',
                                               digits_compute=dp.get_precision('Purchase Price'),
                                               write=['stock.group_stock_manager'],
                                               read=['stock.group_stock_manager'],
                                               help="Product's cost for accounting stock valuation. It is the base price for the supplier price."),
                
                'client_ids': fields.one2many('product.customerinfo', 'product_id', 'Customers'),
                'AS400_default_code' : fields.char('Reference AS400', size=64, select=True)
                }
    
    _defaults = {
        'list_price': 9999.9999,
        'purchase_ok': False
        }

class product_customerinfo(osv.osv):
    _name = "product.customerinfo"
    _description = "Information about a product customer"

    _columns = {
        'name' : fields.many2one('res.partner', 'Customer', required=True,domain = [('customer','=',True)], ondelete='cascade', help="Customer of this product"),
        'product_name': fields.char('Customer Product Name', size=128, help="This customer's product name will be used when printing a request for quotation. Keep empty to use the internal one."),
        'product_code': fields.char('Customer Product Code', size=64, help="This customer's product code will be used when printing a request for quotation. Keep empty to use the internal one."),
        'product_id' : fields.many2one('product.template', 'Product', required=True, ondelete='cascade', select=True),
        'company_id':fields.many2one('res.company','Company',select=1),
    }
    _defaults = {
        'company_id': lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid, 'product.customerinfo', context=c),
    }
    
product_customerinfo()