# -*- coding: utf-8 -*-

from osv import osv, fields
from tools.translate import _

class apem_stock_incoterm(osv.osv):
    _inherit = "stock.incoterms"
    
    _columns = {
                #override to be translatable
                'name': fields.char('Name', size=64, required=True, help="Incoterms are series of sales terms.They are used to divide transaction costs and responsibilities between buyer and seller and reflect state-of-the-art transportation practices.", translate=True),
                }

class apem_sales_order(osv.osv):
    """ This module adapts the sales order module to APEM needs """
    _inherit = "sale.order"

    _columns = {
                #import compatibility
                '_QW_id': fields.integer('Quotewerks ID',size=128),
                
                'sales_assistant': fields.many2one('res.users', 'Sales assistant', states={'draft': [('readonly', False)]}, select=True),
                'customer_service_contact': fields.many2one('res.users', 'Customer service contact', states={'draft': [('readonly', False)]}, select=True),
                
                'introduction_text': fields.text('Introduction text'),
                'conclusion_text': fields.text('Conclusion text'),
                'internal_note': fields.text('Internal note'),
                
                'validity_date': fields.date('Valid until'),
                'commitment_date': fields.date('Commitment Date', help="Date on which delivery of products is to be made."),
                'first_commitment_date': fields.date('First commitment date'),
                
                'minimum_invoicing_price': fields.many2one('sale.order.minimum.invoicing.price', "Minimum invoicing price"),
                
                #used to replace the source with a link in the case it's an opportunity
                'src_opportunity_id': fields.many2one('crm.lead', 'Source Opportunity', context="{'form_view_ref':'crm.crm_case_form_view_oppor'}"),
                
                # for search view only
                'part_numbers': fields.related('abstract_line_ids', 'name', type="char", relation='product.product', string='Part number', store=False)
                }
    
    def create(self, cr, uid, vals, context=None):
        if 'commitment_date' in vals:
                vals['first_commitment_date'] = vals['commitment_date']                
        return super(apem_sales_order,self).create(cr, uid, vals, context)
    
    def write(self, cr, uid, ids, vals, context=None):
        if isinstance(ids, (int, long)):
          ids = [ids]
        
        res = True
        for id in ids:
            if self.browse(cr, uid, id, context).first_commitment_date == False and 'commitment_date' in vals:
                vals['first_commitment_date'] = vals['commitment_date']
                t = super(apem_sales_order,self).write(cr, uid, ids, vals, context)
                del vals['first_commitment_date']    
            else:
                t = super(apem_sales_order,self).write(cr, uid, ids, vals, context)
            if t == False:
                res = False
                    
        return res
    
    def onchange_section_id(self, cr, uid, ids, section_id):
        res = {"domain":{"user_id":False,"sales_assistant":False,"customer_service_contact":False}}
        
        if section_id:
            section = self.pool.get('crm.case.section').browse(cr, uid, section_id)
            
            section_domain = "[('id','in', ["+','.join(map(str,[r.id for r in section.member_ids]))+"])]" or "[]"
            res['domain']['user_id'] = section_domain
            res['domain']['sales_assistant'] = section_domain
            res['domain']['customer_service_contact'] = section_domain
        
        return res
    
    def onchange_partner_id(self, cr, uid, ids, partner_id):
        res = super(apem_sales_order,self).onchange_partner_id(cr, uid, ids, partner_id)
                
        if partner_id:        
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
    
            if partner.section_id:
                res['value'] = res.get('value',{})
                res['value']['section_id'] = partner.section_id.id
                sect_res = self.onchange_section_id(cr, uid, ids, partner.section_id.id)
                res['domain'] = res.get('domain',{})
                res['value'] = res.get('value',{})
                res['warning'] = res.get('warning',{})
                res['domain'].update(sect_res.get('domain',{}))
                res['value'].update(sect_res.get('value',{}))
                res['warning'].update(sect_res.get('warning',{}))
                
                vals = { "user_id": partner.user_id.id or False,
                         "sales_assistant": partner.sales_assistant.id or False,
                         "customer_service_contact": partner.customer_service_contact.id or False}
                res['value'].update(vals)
    
        return res
    
apem_sales_order()

class apem_sales_order_line(osv.osv):
    _inherit = "sale.order.line"
    _order = "order_id, sequence asc, id asc"

    _columns = {
                'delivery_delay': fields.many2one('sale.order.delivery.delay', "Delivery Delay"),
                'internal_note': fields.text('Internal note'),
                'moq': fields.float("Minimal order quantity")
               }
    
    def copy(self, cr, uid, ids, default=None, context=None):
        '''
        Sales layout copy function fails to acknowledge multiple ids.
        This function loops to fix it.
        '''
        return [super(apem_sales_order_line, self).copy(cr, uid, id, default, context) for id in ids]
    
    def copy_line(self, cr, uid, ids, default=None, context=None):
        self.copy(cr, uid, ids, default, context)
        so_id = self.pool.get('sale.order.line').browse(cr,uid,ids[0]).order_id.id
        return {
            'name':_("Sales Orders"),
            'view_mode': 'form',
            'view_type': 'form',
            'res_id' : so_id,
            'res_model': 'sale.order',
            'type': 'ir.actions.act_window'
        }
    
    def product_id_change(self, cr, uid, ids, pricelist, product, qty=0, uom=False, qty_uos=0, uos=False,
            name='', partner_id=False, lang=False, update_tax=True, date_order=False, packaging=False,
            fiscal_position=False, flag=False, context=None):
        
        res = super(apem_sales_order_line, self).product_id_change(cr, uid, ids, pricelist, product, qty=qty,
            uom=uom, qty_uos=qty_uos, uos=uos, name=name, partner_id=partner_id, lang=lang, update_tax=update_tax,
            date_order=date_order, packaging=packaging, fiscal_position=fiscal_position, flag=flag, context=context)

        warning_msgs = ""
        if product != False:
            prod_manager = self.pool.get('product.product')
            cur_prod = prod_manager.browse(cr,uid,product,context=context)
            moq = cur_prod.moq
            if moq != False:
                res['value'] = res.get('value',{})
                res['value']['moq'] = moq                                  
                                  
                if qty < moq:
                    warning_msgs += _("Picking Information ! : The selected quantity is less than the minimum order quantity\n\n")
            qty_ok_packaging = prod_manager.check_packaging_available(cr,uid, qty, [p.qty for p in cur_prod.packaging], context=context)
            if not qty_ok_packaging:
                warning_msgs += _('The quantity you selected can\'t be obtained with any combination of the currently available packagings') + "\n\n"
         
            if 'warning' in res and 'message' in res['warning']:
                res['warning']['message'] += warning_msgs;
            elif warning_msgs != "":
                res['warning'] = {
                    'title': _('Configuration Error !'),
                    'message': warning_msgs}
        return res

apem_sales_order_line()

class apem_make_sale(osv.osv_memory):
    """ Make sale  order for crm """

    _inherit = "crm.make.sale"
    
    def makeOrder(self, cr, uid, ids, context=None):
        res = super(apem_make_sale,self).makeOrder(cr, uid, ids, context=context)
        
        if isinstance(res['res_id'], (int, long)):
            ids = [res['res_id']]
        else:
            ids = res['res_id']            
        
        for cur_SO in ids:
            soManager = self.pool.get('sale.order')
            cur_opp_id = int(soManager.browse(cr,uid,cur_SO, context=context).origin.replace(_('Opportunity: %s')%"", ""))
            soManager.write(cr,uid, [cur_SO], {"src_opportunity_id": cur_opp_id}, context=context)
        
        return res
        
apem_make_sale()